package net.nexcius.gesturerecognizer;

import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;
import java.util.List;

import be.ac.ulg.montefiore.run.jahmm.Hmm;
import be.ac.ulg.montefiore.run.jahmm.ObservationVector;
import be.ac.ulg.montefiore.run.jahmm.OpdfMultiGaussianFactory;
import be.ac.ulg.montefiore.run.jahmm.io.FileFormatException;
import be.ac.ulg.montefiore.run.jahmm.io.ObservationSequencesReader;
import be.ac.ulg.montefiore.run.jahmm.io.ObservationVectorReader;
import be.ac.ulg.montefiore.run.jahmm.learn.BaumWelchScaledLearner;
import be.ac.ulg.montefiore.run.jahmm.learn.KMeansLearner;

import net.nexcius.gesturerecognizer.inputypes.Vec3;
import net.nexcius.gesturerecognizer.support.ResultList;

public class HmmGR {
	private List<Hmm> markovModels;
	private List<String> gestureHandles;
	private int gestureCount = 0;
	
	private int stateCount = 3;
	private double treshold = 1.0E-200;
	private int minVectors = 5;
	
	public HmmGR() {
		markovModels = new ArrayList<Hmm>();
		gestureHandles = new ArrayList<String>();
	}
	
	public void setTreshold(double treshold) {
		this.treshold = treshold;
	}

	
	private Hmm getHmmFromVectors(List<List<ObservationVector>> sequences) {
		KMeansLearner<ObservationVector> kml = new KMeansLearner<ObservationVector>(stateCount, 
				new OpdfMultiGaussianFactory(3), sequences);

		Hmm<ObservationVector> initHmm = kml.iterate();
		
		BaumWelchScaledLearner bwl = new BaumWelchScaledLearner();
		Hmm<ObservationVector> learntHmm = bwl.learn(initHmm, sequences);
		
		return learntHmm;
	}
	
	public void trainGesture(String gestureName, List<List<ObservationVector>> sequences) {
		
		/*System.out.println("Learning");
		for(List<ObservationVector> l : sequences) {
			System.out.println("Size: " + l.size());
		}*/
		
		// Add new gesture
		gestureHandles.add(gestureName);
		markovModels.add(getHmmFromVectors(sequences));
		gestureCount++;
	}
	
	public void trainGestureFromFile(String gestureName, String filename) {
		List<List<ObservationVector>> seqs = null;
		try {
			Reader reader = new FileReader(filename);
			seqs = ObservationSequencesReader.readSequences(new ObservationVectorReader(), reader);
			reader.close();
			
		} catch (IOException e) {
			e.printStackTrace();
		} catch (FileFormatException e) {
			e.printStackTrace();
		}
		
		trainGesture(gestureName, seqs);
	}
	
	
	public ResultList classify(List<ObservationVector> vectors) {
		ResultList rl = new ResultList();
		
		if(vectors.size() < minVectors) {
			return rl;
		}
		
		for(int i = 0; i < gestureCount; i++) {
			double prob = markovModels.get(i).probability(vectors);
			if(prob > treshold) {
				rl.add(gestureHandles.get(i), markovModels.get(i).probability(vectors));
			}
		}
		return rl;
	}
}
