package net.nexcius.gesturerecognizer;

import java.util.ArrayList;
import java.util.List;

import net.nexcius.gesturerecognizer.inputypes.Vec3;
import net.nexcius.gesturerecognizer.support.ResultList;
import net.nexcius.gesturerecognizer.support.VecComparators;

public class LinearGR {
	private List<List<List<Vec3>>> vectorSequences;
	private List<String> gestureHandles;
	private int gestureCount = 0;
	
	//private double treshold = 1.0E-3;
	private double treshold = 0.006;
	private int minVectors = 5;
	
	public LinearGR() {
		vectorSequences = new ArrayList<List<List<Vec3>>>();
		gestureHandles = new ArrayList<String>();
	}
	
	public void setTreshold(double treshold) {
		this.treshold = treshold;
	}
	
	public void trainGesture(String gestureName, List<List<Vec3>> sequences) {

		// Check if existing gesture
		for(int i = 0; i < gestureCount; i++) {
			if(gestureHandles.get(i).equalsIgnoreCase(gestureName)) {
				vectorSequences.get(i).addAll(sequences);
				return;
			}
		}
		
		// Add new gesture
		gestureHandles.add(gestureName);
		vectorSequences.add(sequences);
		gestureCount++;
	}
	
	
	public ResultList classify(List<Vec3> vectors) {
		ResultList rl = new ResultList();
		
		if(vectors.size() < minVectors) {
			return rl;
		}
		
		for(int i = 0; i < gestureCount; i++) {
			double total = 0.0;
			
			for(int j = 0; j < vectorSequences.get(i).size(); j++) {
				total += VecComparators.compareVec3Sequences(vectors, vectorSequences.get(i).get(j));
			}
			
			total /= (double)vectorSequences.get(i).size();
			if(total > treshold) {
				rl.add(gestureHandles.get(i), total);
			}
		}
		return rl;
	}
}
