package net.nexcius.gesturerecognizer.datamanipulation;

import java.util.ArrayList;
import java.util.List;

import be.ac.ulg.montefiore.run.jahmm.ObservationVector;

import net.nexcius.gesturerecognizer.inputypes.NVec3;
import net.nexcius.gesturerecognizer.inputypes.Vec3;

public class NoiseRemoval {
	
	public static List<Vec3> removeNoise(List<Vec3> vectors, int chunkSize, int overlap) {
		
		List<Vec3> returnList = new ArrayList<Vec3>();
		//System.out.println("- " + vectors.size());
		int halfChunkSize = chunkSize/2;
		int step = chunkSize - overlap;
		
		for(int i = halfChunkSize; i < vectors.size()-halfChunkSize; i+=step) {
			int startIndex = i - halfChunkSize;
			float tempX = 0.0f;
			float tempY = 0.0f;
			float tempZ = 0.0f;
			
			for(int j = 0; j < chunkSize; j++) {
				tempX += vectors.get(startIndex + j).getX();
				tempY += vectors.get(startIndex + j).getY();
				tempZ += vectors.get(startIndex + j).getZ();
			}
			
			tempX /= (float)chunkSize;
			tempY /= (float)chunkSize;
			tempZ /= (float)chunkSize;
			
			returnList.add(new Vec3(tempX, tempY, tempZ));
		}
		//System.out.println("- " + returnList.size());
		return returnList;
	}
	
	
	public static List<ObservationVector> removeNoiseOV(List<ObservationVector> vectors, int chunkSize, int overlap) {
		List<ObservationVector> returnList = new ArrayList<ObservationVector>();
		
		int halfChunkSize = chunkSize/2;
		int step = chunkSize - overlap;
		
		for(int i = halfChunkSize; i < vectors.size()-halfChunkSize; i+=step) {
			int startIndex = i - halfChunkSize;
			float tempX = 0.0f;
			float tempY = 0.0f;
			float tempZ = 0.0f;
			
			for(int j = 0; j < chunkSize; j++) {
				tempX += vectors.get(startIndex + j).value(0);
				tempY += vectors.get(startIndex + j).value(1);
				tempZ += vectors.get(startIndex + j).value(2);
			}
			
			tempX /= (float)chunkSize;
			tempY /= (float)chunkSize;
			tempZ /= (float)chunkSize;
			
			returnList.add(new ObservationVector(new double[] {tempX, tempY, tempZ}));
		}
		
		return returnList;
	}
	
	
public static List<NVec3> removeNoiseNVec(List<NVec3> vectors, int chunkSize, int overlap) {
		
		List<NVec3> returnList = new ArrayList<NVec3>();
		//System.out.println("- " + vectors.size());
		int halfChunkSize = chunkSize/2;
		int step = chunkSize - overlap;
		
		for(int i = halfChunkSize; i < vectors.size()-halfChunkSize; i+=step) {
			int startIndex = i - halfChunkSize;
			float tempX = 0.0f;
			float tempY = 0.0f;
			float tempZ = 0.0f;
			
			for(int j = 0; j < chunkSize; j++) {
				tempX += vectors.get(startIndex + j).getX() * vectors.get(startIndex + j).getMagnitude();
				tempY += vectors.get(startIndex + j).getY() * vectors.get(startIndex + j).getMagnitude();
				tempZ += vectors.get(startIndex + j).getZ() * vectors.get(startIndex + j).getMagnitude();
			}
			
			tempX /= (float)chunkSize;
			tempY /= (float)chunkSize;
			tempZ /= (float)chunkSize;
			
			returnList.add(new NVec3(tempX, tempY, tempZ));
		}
		//System.out.println("- " + returnList.size());
		return returnList;
	}
}
