package net.nexcius.gesturerecognizer.inputypes;

/** 
 * Permanently normalized Vec3 class
 * @author Hvard Kindem
 */
public class NVec3 {
    public static float GRAVITY = 9.81f;
    
    private float x;
	private float y;
	private float z;
	private float magnitude;
	
	public NVec3() {
		this.x = 0.0f;
		this.y = 0.0f;
		this.z = 0.0f;
		this.magnitude = 0.0f;
	}
	
	public NVec3(float x, float y, float z) {
		this.magnitude = (float) Math.sqrt((x * x) + (y * y) + (z * z));
		this.x = x / this.magnitude;
		this.y = y / this.magnitude;
		this.z = z / this.magnitude;
	}
	
	public NVec3(NVec3 v) {
		this.x = v.getX();
		this.y = v.getY();
		this.z = v.getZ();
		this.magnitude = v.getMagnitude();
	}
	
	public float getX() {
		return x;
	}

	public float getY() {
		return y;
	}

	public float getZ() {
		return z;
	}
	
	public float getMagnitude() {
		return magnitude;
	}
	
	public float getGravitationalMagnitude() {
	    return Math.abs(magnitude - GRAVITY);
	}
	
	
	/**
	 * Scales this NVec3
	 */
	public void scale(float s) {
		magnitude *= s;
	}
	
	public void addMagnitude(float m) {
		magnitude += m;
	}
	
	
	/**
	 * Returns the dot product
	 * @param v The Vec3 instance to compare to
	 */
	public double dotProduct(NVec3 v) {
		return x * v.getX() + y * v.getY() + z * v.getZ();
	}
	
	
	public double simliarity(NVec3 v) {
		double a = (dotProduct(v) + 1.0) / 2.0;
		double b = 1.0 / (Math.max(magnitude, v.getMagnitude()) / ((magnitude + v.getMagnitude()) / 2.0));
		//System.out.println(a + " - " + b);
		return a * b;
		
		//return (dotProduct(v) + 1.0) / 2.0;
	}
}
