package net.nexcius.gesturerecognizer.inputypes;

import java.math.*;

public class Vec3 {
	public static float GRAVITY = 9.81f;
	
	private float x;
	private float y;
	private float z;
	
	public Vec3() {
		this.x = 0.0f;
		this.y = 0.0f;
		this.z = 0.0f;
	}
	
	public Vec3(float x, float y, float z) {
		this.x = x;
		this.y = y;
		this.z = z;
	}
	
	public Vec3(Vec3 v) {
		this.x = v.getX();
		this.y = v.getY();
		this.z = v.getZ();
	}
	
	public float getX() {
		return x;
	}

	public float getY() {
		return y;
	}

	public float getZ() {
		return z;
	}
	

	
	/**
	 * Returns the length (magnitude) of the instance of this Vec3 class
	 */
	public float length() {
		return (float) Math.sqrt((x * x) + (y * y) + (z * z));
	}
	
	public float getGravitationalMagnitude() {
	    return Math.abs(length() - GRAVITY);
	}
	
	/**
	 * Scales this Vec3
	 */
	public void scale(float s) {
		x *= s;
		y *= s;
		z *= s;
	}
	
	/**
	 * Normalizes this instance of the Vec3 class
	 */
	public void normalize() {
		float len = length();
		x /= len;
		y /= len;
		z /= len;
	}
	
	/**
	 * Returns a new normalized instance of this Vec3
	 */
	public Vec3 getNormalized() {
		float len = length();
		return new Vec3(x/len, y /len, z/len);
	}
	
	/**
	 * Returns the dot product
	 * @param v The Vec3 instance to compare to
	 */
	public double dotProduct(Vec3 v) {
		double eqTop = x * v.getX() + y * v.getY() + z * v.getZ();
		double eqBot = length() * v.length();
		
		return eqTop / eqBot;
	}
	
	
	public double simliarity(Vec3 v) {
		double a = (dotProduct(v) + 1.0) / 2.0;
		double b = 1.0 / (Math.max(length(), v.length()) / ((length() + v.length()) / 2.0));
		//System.out.println(a + " - " + b);
		return a * b;
	}
}
