package net.nexcius.gesturerecognizer.tests;

import java.util.ArrayList;
import java.util.List;

import be.ac.ulg.montefiore.run.jahmm.ObservationVector;
import net.nexcius.gesturerecognizer.FileData;
import net.nexcius.gesturerecognizer.HmmGR;
import net.nexcius.gesturerecognizer.LinearGR;
import net.nexcius.gesturerecognizer.datamanipulation.NoiseRemoval;
import net.nexcius.gesturerecognizer.inputypes.Vec3;
import net.nexcius.gesturerecognizer.support.SequenceLoader;

public class Step2 {
	private static final int CHUNK_SIZE = 3;
	private static final int OVERLAP = 1;
    
	public static void LinearTest() {
		String identifier = "Noise Removal Linear";
    	LinearGR gr = new LinearGR();
    	//gr.setTreshold(0.005);
    	
    	long startTime = 0;
        long endTime = 0;
    	
        List<List<Vec3>> training_left = SequenceLoader.getVecSequences(FileData.training_filenames[0]);
		List<List<Vec3>> training_right = SequenceLoader.getVecSequences(FileData.training_filenames[1]);
		List<List<Vec3>> training_up = SequenceLoader.getVecSequences(FileData.training_filenames[2]);
		List<List<Vec3>> training_down = SequenceLoader.getVecSequences(FileData.training_filenames[3]);
		
    	// Learning Time
    	startTime = System.nanoTime();
    	for(int i = 0; i < training_left.size(); i++) {
    		training_left.set(i, NoiseRemoval.removeNoise(training_left.get(i), CHUNK_SIZE, OVERLAP));
    	}
    	for(int i = 0; i < training_right.size(); i++) {
    		training_right.set(i, NoiseRemoval.removeNoise(training_right.get(i), CHUNK_SIZE, OVERLAP));
    	}
    	for(int i = 0; i < training_up.size(); i++) {
    		training_up.set(i, NoiseRemoval.removeNoise(training_up.get(i), CHUNK_SIZE, OVERLAP));
    	}
    	for(int i = 0; i < training_down.size(); i++) {
    		training_down.set(i, NoiseRemoval.removeNoise(training_down.get(i), CHUNK_SIZE, OVERLAP));
    	}
    	
    	gr.trainGesture(FileData.training_handles[0], training_left);
		gr.trainGesture(FileData.training_handles[1], training_right);
		gr.trainGesture(FileData.training_handles[2], training_up);
		gr.trainGesture(FileData.training_handles[3], training_down);
		endTime = System.nanoTime();
		
		System.out.println(identifier + " - Training Time: " + ((endTime - startTime) / 10E6) + "ms" );
		
		
		// Identification time
		List<List<Vec3>> seqs = new ArrayList<List<Vec3>>();
		
		seqs.add(SequenceLoader.getVecSequences(FileData.test_filenames[0]).get(0));
		seqs.add(SequenceLoader.getVecSequences(FileData.test_filenames[5]).get(0));
		seqs.add(SequenceLoader.getVecSequences(FileData.test_filenames[10]).get(0));
		seqs.add(SequenceLoader.getVecSequences(FileData.test_filenames[15]).get(0));
		seqs.add(SequenceLoader.getVecSequences(FileData.test_filenames[20]).get(0));
		
		startTime = System.nanoTime();
		gr.classify(NoiseRemoval.removeNoise(seqs.get(0), CHUNK_SIZE, OVERLAP));
		gr.classify(NoiseRemoval.removeNoise(seqs.get(1), CHUNK_SIZE, OVERLAP));
		gr.classify(NoiseRemoval.removeNoise(seqs.get(2), CHUNK_SIZE, OVERLAP));
		gr.classify(NoiseRemoval.removeNoise(seqs.get(3), CHUNK_SIZE, OVERLAP));
		gr.classify(NoiseRemoval.removeNoise(seqs.get(4), CHUNK_SIZE, OVERLAP));
		endTime = System.nanoTime();
		
		System.out.println(identifier + " - Classify Time: " + ( ((endTime - startTime) / 5.0) / 10E6) + "ms" );
		
		
		// Accuraccy test
		List<String> expectedResults = new ArrayList<String>();
		List<List<Vec3>> vectorData = new ArrayList<List<Vec3>>();
		
		for(int i = 0; i < FileData.test_filenames.length; i++) {
			expectedResults.add(FileData.test_handles[i]);
			vectorData.add(NoiseRemoval.removeNoise(SequenceLoader.getVecSequences(FileData.test_filenames[i]).get(0), CHUNK_SIZE, OVERLAP));
		}

		AccuracyTest.LinearTest(identifier, gr, expectedResults, vectorData);
	}
	

	public static void HmmTest() {
		String identifier = "Noise Removal HMMs";
		HmmGR gr = new HmmGR();
		//gr.setTreshold(1.0E-200);
    	
    	long startTime = 0;
        long endTime = 0;
        
        List<List<ObservationVector>> training_left = SequenceLoader.getObsVecSequences(FileData.training_filenames[0]);
		List<List<ObservationVector>> training_right = SequenceLoader.getObsVecSequences(FileData.training_filenames[1]);
		List<List<ObservationVector>> training_up = SequenceLoader.getObsVecSequences(FileData.training_filenames[2]);
		List<List<ObservationVector>> training_down = SequenceLoader.getObsVecSequences(FileData.training_filenames[3]);
        
        // Training time		        
		startTime = System.nanoTime();
		for(int i = 0; i < training_left.size(); i++) {
    		training_left.set(i, NoiseRemoval.removeNoiseOV(training_left.get(i), CHUNK_SIZE, OVERLAP));
    	}
    	for(int i = 0; i < training_right.size(); i++) {
    		training_right.set(i, NoiseRemoval.removeNoiseOV(training_right.get(i), CHUNK_SIZE, OVERLAP));
    	}
    	for(int i = 0; i < training_up.size(); i++) {
    		training_up.set(i, NoiseRemoval.removeNoiseOV(training_up.get(i), CHUNK_SIZE, OVERLAP));
    	}
    	for(int i = 0; i < training_down.size(); i++) {
    		training_down.set(i, NoiseRemoval.removeNoiseOV(training_down.get(i), CHUNK_SIZE, OVERLAP));
    	}
		
    	gr.trainGesture(FileData.training_handles[0], training_left);
		gr.trainGesture(FileData.training_handles[1], training_right);
		gr.trainGesture(FileData.training_handles[2], training_up);
		gr.trainGesture(FileData.training_handles[3], training_down);
		endTime = System.nanoTime();
		
		System.out.println(identifier + " - Training Time: " + ((endTime - startTime) / 10E6) + "ms" );
		
		
		// Identification time
		List<List<ObservationVector>> seqs = new ArrayList<List<ObservationVector>>();
		
		seqs.add(SequenceLoader.getObsVecSequences(FileData.test_filenames[0]).get(0));
		seqs.add(SequenceLoader.getObsVecSequences(FileData.test_filenames[5]).get(0));
		seqs.add(SequenceLoader.getObsVecSequences(FileData.test_filenames[10]).get(0));
		seqs.add(SequenceLoader.getObsVecSequences(FileData.test_filenames[15]).get(0));
		seqs.add(SequenceLoader.getObsVecSequences(FileData.test_filenames[20]).get(0));

		startTime = System.nanoTime();
		gr.classify(NoiseRemoval.removeNoiseOV(seqs.get(0), CHUNK_SIZE, OVERLAP));
		gr.classify(NoiseRemoval.removeNoiseOV(seqs.get(1), CHUNK_SIZE, OVERLAP));
		gr.classify(NoiseRemoval.removeNoiseOV(seqs.get(2), CHUNK_SIZE, OVERLAP));
		gr.classify(NoiseRemoval.removeNoiseOV(seqs.get(3), CHUNK_SIZE, OVERLAP));
		gr.classify(NoiseRemoval.removeNoiseOV(seqs.get(4), CHUNK_SIZE, OVERLAP));
		endTime = System.nanoTime();
		
		System.out.println(identifier + " - Classify Time: " + ( ((endTime - startTime) / 5.0) / 10E6) + "ms" );
		
		
		// Accuraccy test
		List<String> expectedResults = new ArrayList<String>();
		List<List<ObservationVector>> vectorData = new ArrayList<List<ObservationVector>>();
		
		for(int i = 0; i < FileData.test_filenames.length; i++) {
			expectedResults.add(FileData.test_handles[i]);
			vectorData.add(SequenceLoader.getObsVecSequences(FileData.test_filenames[i]).get(0));
		}		
		
		for(int i = 0; i < vectorData.size(); i++) {
			vectorData.set(i, NoiseRemoval.removeNoiseOV(vectorData.get(i), CHUNK_SIZE, OVERLAP));
		}
	
		AccuracyTest.HmmTest(identifier, gr, expectedResults, vectorData);
	}
}

